import { CharacterGender, CharacterRace } from '@prisma/client'
import { Server } from 'socket.io'

import prisma from '#application/prisma'
import { TSocket } from '#application/types'
import characterRepository from '#repositories/characterRepository'

export default class CharacterTypeCreateEvent {
  constructor(
    private readonly io: Server,
    private readonly socket: TSocket
  ) {}

  public listen(): void {
    this.socket.on('gm:characterType:create', this.handleEvent.bind(this))
  }

  private async handleEvent(data: undefined, callback: (response: boolean, characterType?: any) => void): Promise<void> {
    try {
      const character = await characterRepository.getById(this.socket.characterId as number)
      if (!character) return callback(false)

      if (character.role !== 'gm') {
        return callback(false)
      }

      const newCharacterType = await prisma.characterType.create({
        data: {
          name: 'New character type',
          gender: CharacterGender.MALE,
          race: CharacterRace.HUMAN
        }
      })

      callback(true, newCharacterType)
    } catch (error) {
      console.error('Error creating character type:', error)
      callback(false)
    }
  }
}
